import josx.platform.rcx.*;

public class MyRobotS2 {
	private int darkLightS1, darkLightS3;
	private final int DARKLINE_DATACOUNT = 1000;
	private final int ROTATION_POWER = 6;
	private final int DRIVE_POWER = 7;

	public static void main(String[] args) throws InterruptedException {
		new MyRobotS2();
	}

	public MyRobotS2() throws InterruptedException {
		// Lichtsensor initialisieren
		Sensor.S1.setTypeAndMode(SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_PCT);
		Sensor.S3.setTypeAndMode(SensorConstants.SENSOR_TYPE_LIGHT,
				SensorConstants.SENSOR_MODE_PCT);

		Sensor.S1.setPreviousValue(0);
		Sensor.S3.setPreviousValue(0);

		Sensor.S1.activate();
		Sensor.S3.activate();

		setNormalDarkLight();
		boolean onS1, onS3, drive = false;

		while (!Button.RUN.isPressed()) {
			onS1 = isOnLineS1();
			onS3 = isOnLineS3();

			// geradeaus fahren, wenn auf der Linie
			if (!drive && onS1 && onS3) {
				drive = true;
				MotorsPower(DRIVE_POWER);
				Motor.A.forward();
				Motor.C.forward();
			}

			// Linie wurde verlassen
			if (drive && (!onS1 || !onS3)) {
				drive = rotateUntilOnLine(onS1, onS3);
			}
		}

		Sensor.S1.passivate();
		Sensor.S3.passivate();

		Motor.A.flt();
		Motor.C.flt();

		Button.RUN.waitForPressAndRelease();
	}

	// Roboter dreht sich, bis er wieder auf der Linie ist
	private boolean rotateUntilOnLine(boolean onS1, boolean onS3) {
		MotorsPower(ROTATION_POWER);

		// nur Sensor 1 verliert Linie
		if (!onS1 && onS3) {
			Motor.A.forward();
			Motor.C.flt();
			sleep(120);
			Motor.C.backward();
			while (!isOnLineS1() && !Button.RUN.isPressed());
			return false;
		}

		// nur Sensor 3 verliert Linie
		if (onS1 && !onS3) {
			Motor.C.forward();
			Motor.A.flt();
			sleep(120);
			Motor.A.backward();
			while (!isOnLineS3() && !Button.RUN.isPressed());
			return false;
		}

		// weder Sensor 1 noch Sensor 2 sind auf der Linie
		if (!onS1 && !onS3) {
			// zurueckfahren, damit mind.
			// ein Sensor die Linie sieht
			Motor.A.backward();
			Motor.C.backward();
		}

		return true;
	}

	private boolean isOnLineS1() {
		return Sensor.S1.readValue() <= darkLightS1;
	}

	private boolean isOnLineS3() {
		return Sensor.S3.readValue() <= darkLightS3;
	}

	// normales dunkles Licht ermitteln
	private void setNormalDarkLight() {
		int sumS1 = 0, sumS3 = 0;
		for (int i = 0; i <= DARKLINE_DATACOUNT; i++) {
			sumS1 += Sensor.S1.readValue();
			sumS3 += Sensor.S3.readValue();
		}
		darkLightS1 = sumS1 / DARKLINE_DATACOUNT + 5;
		darkLightS3 = sumS3 / DARKLINE_DATACOUNT + 5;
	}

	// Geschwindigkeit der beider Motoren setzen
	private void MotorsPower(int p) {
		Motor.A.setPower(p);
		Motor.C.setPower(p);
	}

	// Pause
	private void sleep(int aMilliseconds) {
		try {
			Thread.sleep(aMilliseconds);
		} catch (Exception e) {
		}
	}

}